﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using MvcTempDemo1.Models;

namespace MvcTempDemo1.Binders {
    public class EmployeeBinder : DefaultModelBinder {
        
        public override object BindModel(ControllerContext controllerContext, ModelBindingContext bindingContext) {

            var model = base.BindModel(controllerContext, bindingContext) as EmployeeModel; //Bind most of the model using the built-in binder, this will bind all primitives for us 
            
            const string magicString = "Offices";  //TODO Use of magic strings

            var offices = bindingContext.ValueProvider.GetValue(magicString); //Get the posted value for offices

            if (offices != null && !string.IsNullOrEmpty(offices.AttemptedValue)) { //Check we have a value for offices before we proceed

                bindingContext.ModelState.Remove(magicString); //Remove binding conversion errors for offices, as we are going to deal with binding offices ourselves

                try {
                    //Create a list of offices based on the comma delimited string of posted OfficeId's
                    model.Offices = new List<Office>(
                                                        offices.AttemptedValue.Split(",".ToCharArray())
                                                        .Select(id => new Office() { OfficeId = new Guid(id) })
                                                        .ToList()
                                                    );
                }
                catch (FormatException ex) { //Catch if the posted offices are not posted as a comma delimited string of Guids
                    bindingContext.ModelState.AddModelError(magicString, ex); //Add an error to the model state, used for ModelState.IsValid and Html error helpers
                }
                catch (Exception ex) { //Unexpected exception
                    throw ex;
                }
            }

            return model;
        }

    }
}
